/* $Id: JSAFE_SecureRandom.java,v 1.51 2005/06/21 23:29:05 cscott Exp $ */

/**
 * Copyright (c) RSA Security Inc., 1997-2005.  All rights reserved.
 * This work contains proprietary, confidential, and trade secret
 * information of RSA Security Inc.  Use, disclosure or reproduction
 * without the express written authorization of RSA Security Inc. is
 * prohibited.
 */

package com.rsa.jsafe;

import java.security.SecureRandom;
import java.security.NoSuchAlgorithmException;
import java.io.Serializable;

/**
 * This class defines secure pseudorandom number generation,
 * as well as random number generation in hardware. For more
 * information about pseudorandom number generation, refer to
 * the <em>Crypto-J Developer's Guide</em>.
 * <p>
 * <code>JSAFE_SecureRandom</code> extends the
 * <code>java.security.SecureRandom</code> class
 * so that it can be used to replace the Sun JavaSoft classes. All
 * Crypto-J classes that use this class actually request a
 * <code>java.security.SecureRandom</code> Java class, so that you can use
 * the standard Java classes instead of <code>java.security.SecureRandom</code>.
 * <p>
 * See <em><a href="{@docRoot}/html/introtext.html">
 * Overview of Crypto-J </a></em> for background and reference material
 * on using and understanding Crypto-J.
 * <p>
 * Copyright &copy; RSA Security Inc., 1997-2005.  All rights reserved.
 */

public abstract class JSAFE_SecureRandom
   extends SecureRandom
   implements Cloneable, Serializable
{
  private String theDevice;
  private String[] theDeviceList;

  // library-wide (global) seed generator
  private static SecureRandom seeder = null;

  private static final boolean[] strictHW = { true, false };
  private static final String[] interfaceListHW = {
    "com.rsa.jsafe.JA_AlgaeRandom",
    "com.rsa.jsafe.JA_AlgaeDigest"
  };

  private static final boolean[] strictChain = { true };
  private static final String[] interfaceListChain = {
    "com.rsa.jsafe.JA_AlgaeChainDigestRandom"
  };

  private static final String fullyQualClassName =
    "com.rsa.jsafe.JSAFE_SecureRandom";

  // All Cryptograpic Transformation classes perform
  // this check when they enter getInstance, let's
  // have it in a common place
  final static void checkIntegrity () {
    CryptoJ.checkIntegrity ();
  }

  // convenience method to test library fips mode
  final static boolean isInFIPSMode () {
    return CryptoJ.isInFIPSMode ();
  }

  JSAFE_SecureRandom ()
  {
    // The superclass is SecureRandom. In JDK 1.1, calling the constructor
    // without a seed, which would be done automatically, triggers the autoseed.
    // Prevent this by use of this "dummy" seed. On JDK 1.1 the super class
    // constructor will call setSeed with this length-zero byte array which
    // setSeed will ignore.
    super(new byte[0]);

    // Without this call, on JDK 1.1 the super class default constructor
    // would have autoseeded with a 20 bytes of seed and would have
    // called setSeed method which in turn calls seed. At construction
    // time an attempt to digest the seed would result in a
    // NullPointerException as the digest is still null. To prevent
    // this we called super with our array that setSeed will ignore.

  }

  /**
   * Instantiates the global seed generator if it hasn't beeen done already.
   * Synchronized methods are expensive so this will be called by an unsynchronized
   * one only if the instance is null.
   * <p>
   */
  private synchronized static SecureRandom getSeederSync()
  {
    // We test again
    if (seeder == null)
       seeder = new SecureRandom ();
    return seeder;
  }

  /*
   * Returns the global seed generator. Instantiates it if it hasn't beeen done already
   * <p>
   * @return the Crypto-J gloabal seed generator
   */
  protected static SecureRandom getSeeder()
  {
    // Using the double checked locking pattern for
    // multi-threaded safety but minimal overhead.
    if (seeder == null) {
        // call the synchronized version
       seeder = getSeederSync ();
    }
    return seeder;
  }

  /*
   * Returns numBytes of seed. It obtains bytes of seed from the seed
   * generation algorithm that the default SecureRandom uses to seed
   * itself.
   * <p>
   * @param numBytes number of bytes of seed to generate
   */
  protected static byte[] getRandomSeed(int numBytes)
  {
    // Normally we would call seeder.generateRandomBytes(numNytes)
    // but that is not available on JRE 1.1 which we still support.
    // We would have to do it via reflection using either a
    // PrivilegedAction or a PrivilegedExceptionAction. We will use
    // nextBytes instead. This may trigger autoseed the first time
    // but we let the user have access to the so they can seed it
    // or add reseed it at they see fit.
    byte[] tempBytes = new byte[numBytes];
    getSeeder().nextBytes(tempBytes);
    return tempBytes;
  }

  /**
   * Builds an object that performs the <code>transformation</code> on the
   * given <code>device</code>. Note that this instance of
   * <code>JSAFE_SecureRandom</code> has not been seeded.
   * A call to the <code>setSeed()</code> method, or other seeding methods,
   * will seed the <code>JSAFE_SecureRandom</code> object. If a call is not made
   * to <code>setSeed()</code>, the first call to the <code>nextBytes()</code>
   * method will force the <code>SecureRandom</code> object to seed itself.
   * <pre>
   * Note:  Due to a change in <code>java.security.SecureRandom</code> (the base class for
   *        <code>JSAFE_SecureRandom</code>) in JDK 1.2, the <code>getInstance</code> method must
   *        be changed. In earlier versions of Crypto-J, the <code>JSAFE_SecureRandom</code>
   *        class had a typical <code>getInstance</code> method that took two Strings as arguments;
   *        this was not a problem because, at that time, the base class, <code>SecureRandom</code>,
   *        did not have a <code>getInstance</code> method. In Crypto-J versions 2.1 and later, the
   *        <code>JSAFE_SecureRandom.getInstance</code> method returns an instance of
   *        <code>JSAFE_SecureRandom</code> cast as a <code>SecureRandom</code> class, and throws
   *        <code>java.security.NoSuchAlgorithmException</code>.
   *
   *        If you are updating code from Crypto-J version 2.0 or earlier, see the section
   *        "Updating JSAFE_SecureRandom.getInstance()" under "Random Number Generation" of the
   *        <em>Crypto-J Developer's Guide</em> for more information.
   * </pre>
   * The value of transformation must be one of the following:
   * <ul>
   * <li>"MD5Random"
   * <li>"SHA1Random"
   * <li>"FIPS186Random"
   * <li>"X931Random-<em>s</em>"
   * <li>"HWRandom/<em>digest</em>"
   * </ul>
   * The value of <em>s</em> when used with "X931Random" is the number of streams, a value from
   * 1 to 6.
   *
   * <pre>
   * Note: "FIPS186Random" is a general purpose FIPS 140-2 compliant pseudo random number
   *       generator based on the on the FIPS 186 standard.
   * </pre>
   *
   * <pre>
   * Note:  Using "X931Random" will produce pseudo-random results in compliance with
   *        the X9.31 standard. If a hardware random number generator is not available,
   *        this software pseudo-random number generator is required to generate
   *        strong RSA key pairs. This pseudo-random number generator also requires a
   *        minimum number of seed bytes per stream.
   * </pre>
   * The value of <em>digest</em> when used with "HWRandom" must be one of the following:
   * <ul>
   * <li>"MD2"
   * <li>"MD5"
   * <li>"SHA1"
   * <li>"NoDigest"
   * </ul>
   * <pre>
   * Note:  The digest value is for whitening the hardware results. To produce
   *        unwhitened bytes, use "NoDigest". <em>Whitening</em> the output of a random number
   *        generator means applying a post-processing algorithm to reduce patterns in
   *        the hardware bits and make them less predictable. The advantage of
   *        performing whitening in software as well as hardware is that an attacker
   *        must modify the hardware and the software to make the HRNG leak secret
   *        information. If you plan to use the random numbers directly, you should use
   *        "MD2", "MD5", or "SHA1" as the "HWRandom" digest value to apply additional
   *        whitening. If you are seeding a pseudo-random number generator,
   *        you can use "NoDigest" for optimal performance.
   * </pre>
   * The device value is as follows: <em>choice1[/choice2[...[/choicen]]]</em>
   * where the choices for device value are as follows:
   * <ul>
   * <li>"Java"
   * <li>"Native"
   * <li>"Intel"
   * </ul>
   * For "HWRandom", the only hardware device that is currently available is
   * "Intel". The "Intel" device can only perform "HWRandom".
   * For information on generating random numbers using the "Intel" device,
   * see the <em>Intel Security Hardware User's Guide</em>.
   * <p>
   * Crypto-J tries to instantiate a class using the first choice; if it cannot, it tries
   * the other choices.
   * <pre>
   * Note:  Do not attempt to use hardware versions of Crypto-J classes unless you are
   *        very familiar with the hardware. See the "Random Number Generation" section
   *        of the <em>Crypto-J Developer's Guide</em>, which describes hardware usage,
   *        its benefits, and its problems, as well as the <em>Intel Security Hardware User's
   *        Guide</em>.
   *
   * Note:  In JDK 1.1, <code>java.security.SecureRandom</code> autoseeds on instantiation
   *        if a seed is not provided. <code>JSAFE_SecureRandom</code> does not autoseed on
   *        instantiation, but does offer an autoseed() method for pseudo-random number generators.
   *        Autoseeding is not needed for hardware random number generators such as Intel's.
   * </pre>
   * Examples
   * <p>
   * The first example demonstrates how to build a pseudo-random object in software:
   * <pre>
   * <code> JSAFE_SecureRandom md5Random = (JSAFE_SecureRandom)
   *        JSAFE_SecureRandom.getInstance ("MD5Random", "Native/Java");
   * </code></pre>
   * The second example demonstrates how to build a random object using the HWRandom
   * algorithm. This object can later be used to seed a Crypto-J pseudo-random number
   * generator (PRNG). To get a <code>JSAFE_SecureRandom</code> object, you should
   * cast the result with <code>JSAFE_SecureRandom</code>.
   * <pre>
   * <code>  JSAFE_SecureRandom intelRandom = (JSAFE_SecureRandom)
   *        JSAFE_SecureRandom.getInstance("HWRandom/SHA1" , "Intel");
   * </code></pre>
   * If the Intel hardware is not available, Crypto-J throws a <code>NoSuchAlgorithmException</code>.
   * If the Intel hardware is available, Crypto-J creates an instance of <code>JSAFE_SecureRandom</code>
   * that can perform Intel random number generation. If something goes wrong after this
   * instance has been created, Crypto-J throws an <code>IntelException</code>. See the <em>Intel Security
   * Hardware User's Guide</em> for further information.
   * </p>
   * @see #setSeed
   * @see #seed
   * @see #extraSeed
   * @see #nextBytes
   * @see #generateRandomBytes
   *
   * @param transformation The representation of the desired operation
   * (for example, "MD5Random").
   * @param device A list of devices used to build the object
   * (for example, "Java", "Native/Java").
   * @return A new <code>JSAFE_SecureRandom</code> object that performs the
   * <code>transformation</code> on the given <code>device</code>.
   * @exception NoSuchAlgorithmException If the device or devices cannot
   * perform the designated algorithm, or if the <code>transformation</code>
   * contains parameters that do not work.
   */
  public static SecureRandom getInstance
    (String transformation,
     String device)
    throws NoSuchAlgorithmException
  {
    // Crypto-J must not have entered an inoperable state.
    JSAFE_Object.checkIntegrity ();

    // There has to be device and transformation.
    if (device == null)
      throw new NoSuchAlgorithmException
        ("Cannot instantiate: no device given.");
    if (transformation == null)
      throw new NoSuchAlgorithmException
        ("Cannot instantiate: no transformation given.");

    // Parse the device list.
    String[] deviceNames = JA_ParseList.parseElements (device);
    JSAFE_DeviceBuilder[] deviceObjects =
      new JSAFE_DeviceBuilder[deviceNames.length];

    // Parse the transformation.
    String[] transformationNames = JA_ParseList.parseElements (transformation);

    // Scroll through the list of devices until an object is built.
    for (int index = 0; index < deviceNames.length; index++) {
      // Get the DeviceBuilder object.
      if (deviceObjects[index] == null)
        deviceObjects[index] = JA_ParseDevice.getDeviceObject
          (deviceNames[index]);

      try {
        // Call the getInstance with the builder.
        JSAFE_SecureRandom returnObject =
          JSAFE_SecureRandom.getInstance
          (transformationNames, deviceNames, deviceObjects[index],
           deviceObjects);
        // If this did not return null, we have our object.
        if (returnObject == null)
          continue;
        // Now which device built it?
        returnObject.theDevice = deviceObjects[index].getDevice ();
        returnObject.theDeviceList = deviceObjects[index].getDeviceList ();
        // Cast the return object to SecureRandom because of the JDK 1.2
        //   change to SecureRandom.
        return returnObject;
      } catch (JSAFE_InvalidParameterException paramException) {
        // Try the next device.
      }
    }

    // If we scrolled through the device list and could not build an
    //   object, throw an exception.
    throw new NoSuchAlgorithmException
      ("A JSAFE_SecureRandom object of " + transformation +
       " is not available on any of the devices. (" + device + ")");
  }

  /**
   * Build an object that performs the operation represented by the array
   * <code>transformationNames</code> on the device or devices specified by
   * <code>deviceNames</code>.
   * <p>The <code>deviceBuilder</code> is the current device trying to build
   * the object. If this device builder wants to look at other devices, it
   * can through <code>deviceNames</code> and <code>deviceObjects</code>.
   * <p>
   * @param transformationNames An array of Strings specifying the components
   * of the transformation.
   * @param deviceNames An array of Strings specifying the user-requested
   * devices.
   * @param deviceBuilder The device builder to use.
   * @param deviceObjects An array of device builder objects (elements may
   * be <code>null</code> because not yet instantiated).
   * @return A new <code>JSAFE_SecureRandom</code> object that performs the
   * transformation.
   * @exception JSAFE_InvalidParameterException If the transformation
   * contains parameters that do not work.
   */
  private static JSAFE_SecureRandom getInstance
    (String[] transformationNames,
     String[] deviceNames,
     JSAFE_DeviceBuilder deviceBuilder,
     JSAFE_DeviceBuilder[] deviceObjects)
    throws JSAFE_InvalidParameterException
  {
    // Check to see if we have only a random.
    if (transformationNames.length == 1) {
      // Instantiate the components from the DeviceBuilder
      Object[] theObjects = deviceBuilder.buildObjects
        (transformationNames, null, JA_ParseTransformation.SECURE_RANDOM,
         strictChain, interfaceListChain, fullyQualClassName, deviceNames,
         deviceObjects);

      // Could the device build the objects?
      if (theObjects == null)
        return (null);

      // Load up the params.
      int[] params = null;

      params = JA_ParseList.getParameterList (transformationNames[0]);
      ((JA_AlgaeChainDigestRandom)(theObjects[0])).
          setInstantiationParameters (params);

      // allows X931 and X962 or X982 to have a special case
      // beware of adding another random with a different name
      String algorithm = ((JA_AlgaeChainDigestRandom)theObjects[0]).getAlgorithm ();
      if (algorithm.startsWith ("X9") ||
          algorithm.equals("FIPS186Random"))
        return ((JSAFE_SecureRandom)theObjects[0]);

      return (new JG_ChainDigestRandom
        ((JA_AlgaeChainDigestRandom)theObjects[0]));
    }

    // Now, check for 2 transformation names.
    // Any other number of algs is an error.
    if (transformationNames.length != 2)
      return (null);

    int[] params = JA_ParseList.getParameterList
      (transformationNames[0]);

    Object[] theObjects = deviceBuilder.buildObjects
      (transformationNames, null, JA_ParseTransformation.SECURE_RANDOM,
       strictHW, interfaceListHW, fullyQualClassName, deviceNames,
       deviceObjects);

    // Could the device build the objects?
    if (theObjects == null)
      return (null);

    // Put everything together at the JG layer
    return (new JG_DigestedRandom
      ((JA_AlgaeRandom)theObjects[0],
       (JA_AlgaeDigest)theObjects[1]));
  }

  /**
   * Returns the name of the device of record.
   * Possible <em>device</em> values are:
   * <ul>
   * <li>Java
   * <li>Native
   * <li><em>name</em> of the specified device
   * </ul>
   * <p>
   * @return The device name.
   */
  public String getDevice ()
  {
    return (theDevice);
  }

  /**
   * Returns a <code>String</code> array that describes
   * all the devices used to execute the <em>transformation</em>. Because a
   * <em>transformation</em> often consists of component algorithms, the
   * implementation can consist of several component objects.
   * Returns the names of the devices used by each component.
   * <p>
   * @return A <code>String</code> array that describes the device used for a
   * specific component.
   */
  public String[] getDeviceList ()
  {
    String[] returnValue = new String[theDeviceList.length];
    for (int index = 0; index < theDeviceList.length; index++)
      returnValue[index] = theDeviceList[index];
    return (returnValue);
  }

  /**
   * Returns a new <code>int</code> array containing the algorithm's parameters.
   * This may be an array of length <code>0</code>.
   * <p>
   * @return A new <code>int</code> array that contains the parameters.
   */
  public int[] getAlgorithmParameters ()
  {
    return (new int[0]);
  }

  /**
   * Returns the standard algorithm name.
   * <p>
   * @return A <code>String</code> that describes the algorithm.
   */
  public abstract String getAlgorithm ();

  /**
   * Seeds the pseudorandom number generator. This method uses the entire
   * input buffer, and seeds an object. It uses all the bytes of the
   * <code>seedBytes</code> array and can be called at any time. The seed method
   * "adds" the new seed bytes, but does not replace any old seeding. Each
   * call to <code>seed()</code>, <code>autoseed()</code>, or
   * <code>generateRandomBytes()</code> updates the internal state of an object.
   * <p>
   * @param seedBytes A <code>byte</code> array that alters the internal state
   * of the PRNG.
   */
  public abstract void seed
    (byte[] seedBytes);

  /**
   * Generates seed bytes and uses them to seed an object.
   * Provides "extra" seed bytes for PRNGs that use two kinds of seed,
   * such as an X9.31 random seed.
   * <p>
   * To generate a random seed that complies with the X9.31 specification,
   * there must be two independent streams of seeding.
   * A standard seed is supplied by calling <code>seed()</code>,
   * <code>autoseed()</code>, or <code>generateRandomBytes()</code>,
   * and a user-supplied seed, by calling <code>extraSeed()</code>.
   * Both seeds are required to maintain compliance with the X9.31 specification.
   * If the underlying algorithm does not use two kinds of seed, this
   * method will simply use these seed bytes as regular seed. This method
   * uses the entire input buffer.
   * <p>
   * The size, in bytes, of the <code>extraSeedBytes</code> passed in to the
   * <code>extraSeed()</code> method must be between 20 and 64 times the
   * number of streams.
   * <p>
   * @see #seed
   * @see #setSeed
   * @param extraSeedBytes The input, used as extra seed bytes if the
   * algorithm uses two distinct types of seed.
   * @exception JSAFE_InputException If the extra seed does not match the
   * algorithm's expectations.
   */
  public void extraSeed
    (byte[] extraSeedBytes)
    throws JSAFE_InputException
  {
    // Subclasses that use extra seed should override this method.
    seed (extraSeedBytes);
  }

  /**
   * This method generates seed bytes and uses them to seed an object.
   * The technique used is based on the autoseeding from
   * <code>java.security.SecureRandom</code>.
   * <p>
   * This method may take four or five seconds.
   * <pre>
   * Note:  JavaSoft provides the following cautionary statement regarding their
   *        autoseeding algorithm: We attempt to provide sufficient seed bytes to
   *        completely randomize the internal state of the generator (20 bytes). Note,
   *        however, that our seed generation algorithm has not been thoroughly studied
   *        or widely deployed. It relies on counting the number of times that the calling
   *        thread can yield while waiting for another thread to sleep for a specified
   *        interval.
   * </pre>
   */
  public abstract void autoseed ();

  /**
   * Seeds the pseudorandom number generator with the <code>seedLong</code>
   * (8-byte integer) value.
   * <p>
   * Reseeds this random object, using the eight bytes contained
   * in the <code>seedLong</code> parameter. The given seed supplements,
   * rather than replaces, the existing seed. Thus, repeated calls
   * are guaranteed never to reduce randomness.
   * <p>
   * @see #seed
   * @param seedLong A <code>long</code> that alters the internal state
   * of the PRNG.
   */
  public void setSeed
    (long seedLong)
  {
    /*
     * Ignore call from super constructor (as well as any other calls
     * unfortunate enough to be passing 0).  It's critical that we
     * ignore call from superclass constructor, as the digest has not
     * yet been initialized at that point and we want to defer seeding.
     */
    if (seedLong != 0)
      setLongSeed (seedLong);
  }

  /**
   * Seeds the pseudorandom number generator with the long (8-byte integer)
   * value.
   * <p>
   * @param seedLong A <code>long</code>, the bits of which are used to alter
   * the internal state of the PRNG.
   */
  private void setLongSeed
    (long seedLong)
  {
    byte[] getLong = new byte[8];
    for (int index = 0; index < 8; index++) {
      getLong[index] = (byte)(seedLong & 0xff);
      seedLong = seedLong >>> 8;
    }
    seed (getLong);
    JSAFE_Obfuscator.overwrite (getLong);
    return;
  }

  /**
   * Seeds the pseudorandom number generator. This method uses the entire
   * input buffer.
   * <p>
   * @see #seed
   * @param seedBytes A <code>byte</code> array that alters the internal
   * state of the PRNG.
   */
  public void setSeed
    (byte[] seedBytes)
  {
    /*
     * Ignore call from super constructor (as well as any other calls
     * unfortunate enough to be passing 0 or null).  It's critical that we
     * ignore call from superclass constructor, as the digest has not
     * yet been initialized at that point and we want to defer seeding.
     */
    if (seedBytes != null && seedBytes.length != 0)
      seed (seedBytes);
  }

  /**
   * Fills the input buffer with pseudorandom bytes.
   * <p>
   * @see #setSeed
   * @param bytes A buffer that is filled entirely with pseudorandom bytes.
   */
  public void nextBytes
    (byte[] bytes)
  {
    generateRandomBytes (bytes, 0, bytes.length);
  }

  /**
   * Generates a pseudorandom <code>short</code> (2-byte integer) value.
   * <p>
   * @return A pseudorandom <code>short</code> value.
   */
  public short nextShort ()
  {
    byte[] randomBytes = new byte[2];
    generateRandomBytes (randomBytes, 0, 2);
    int returnValue =
       (randomBytes[0] & 0xff) |
      ((randomBytes[1] & 0xff) << 8);

    JSAFE_Obfuscator.overwrite (randomBytes);
    return ((short)returnValue);
  }

  /**
   * Generates a pseudorandom <code>int</code> (4-byte integer) value.
   * <p>
   * @return A pseudorandom <code>int</code> value.
   */
  public int nextInt ()
  {
    byte[] randomBytes = new byte[4];
    generateRandomBytes (randomBytes, 0, 4);
    int returnValue =
       (randomBytes[0] & 0xff) |
      ((randomBytes[1] & 0xff) << 8) |
      ((randomBytes[2] & 0xff) << 16) |
      ((randomBytes[3] & 0xff) << 24);

    JSAFE_Obfuscator.overwrite (randomBytes);
    return (returnValue);
  }

  /**
   * Generates a pseudorandom <code>long</code> (8-byte integer) value.
   * <p>
   * @return A pseudorandom <code>long</code> value.
   */
  public long nextLong ()
  {
    byte[] randomBytes = new byte[8];
    generateRandomBytes (randomBytes, 0, 8);
    long returnValue
      = (long)(randomBytes[0] & 0xff)
        | (((long)(randomBytes[1] & 0xff)) << 8)
        | (((long)(randomBytes[2] & 0xff)) << 16)
        | (((long)(randomBytes[3] & 0xff)) << 24)
        | (((long)(randomBytes[4] & 0xff)) << 32)
        | (((long)(randomBytes[5] & 0xff)) << 40)
        | (((long)(randomBytes[6] & 0xff)) << 48)
        | (((long)(randomBytes[7] & 0xff)) << 56);
    JSAFE_Obfuscator.overwrite (randomBytes);
    return (returnValue);
  }

  /**
   * Generates a pseudorandom <code>double</code>
   * (8-byte floating-point number) value. This method builds only
   * positive values.
   * <p>
   * @return A pseudorandom <code>double</code> value.
   */
  public double nextDouble ()
  {
    long doubleAsLong = nextLong ();

    // Make sure it is positive, non-zero and less than inf.
    doubleAsLong &= 0x7fffffffffffffffL;
    if (doubleAsLong == 0L)
      return (nextDouble ());
    if ((doubleAsLong & 0x7ff0000000000000L) == 0x7ff0000000000000L)
      return (nextDouble ());

    double theValue = Double.longBitsToDouble (doubleAsLong);
    doubleAsLong = 0;
    return (theValue);
  }

  /**
   * Generates a pseudorandom <code>float</code> (4-byte floating-point number)
   * value. This method builds only positive values.
   * <p>
   * @return A pseudorandom <code>float</code> value.
   */
  public float nextFloat ()
  {
    int floatAsInt = nextInt ();

    // Make sure it is positive, non-zero and less than inf.
    floatAsInt &= 0x7fffffffffffffffL;
    if (floatAsInt == 0)
      return (nextFloat ());
    if ((floatAsInt & 0x7f800000) == 0x7f800000)
      return (nextFloat ());

    float theValue = Float.intBitsToFloat (floatAsInt);
    floatAsInt = 0;
    return (theValue);
  }

  /**
   * Generates pseudorandom bytes, returning them in a new <code>byte</code>
   * array of length <code>numberOfBytes</code>.
   * <p>
   * @param numberOfBytes The number of pseudorandom bytes to generate.
   * @return A new <code>byte</code> array containing the pseudorandom output.
   */
  public byte[] generateRandomBytes (int numberOfBytes)
  {
    // Create the byte array to return.
    byte[] randomBytes = new byte[numberOfBytes];

    // Call the other generate to avoid code duplication.
    generateRandomBytes (randomBytes, 0, numberOfBytes);
    return (randomBytes);
  }

  /**
   * Generates pseudorandom bytes, placing them into the given buffer.
   * This method generates <code>numberOfBytes</code> random bytes and places
   * them into the <code>byte</code> array <code>randomOutput</code>,
   * beginning at <code>offset</code>.
   * <p>
   * This method is used as the basis of all random entities returned by
   * this class (except seed bytes).
   * <p>
   * @param randomOutput The buffer where the pseudorandom output is placed.
   * @param offset The offset into <code>randomOutput</code> where the writing
   * begins.
   * @param numberOfBytes The number of pseudorandom bytes to generate.
   */
  public abstract void generateRandomBytes
    (byte[] randomOutput, int offset, int numberOfBytes);

  /*
   * Performs the FIPS 140 required Continuous Random Number Generation Check.
   * Compares the current block with the previous. The expected result is for
   * the blocks to be different. If the blocks are identical
   * it throws an unchecked SecurityException.
   * <p>
   * @param previousBlock the previously generated block
   * @param previousOffset where previous data starts
   * @param currentBlock the current block being generated
   * @param currentOffset where current data starts
   * @param count how many bytes to compare
   */
  protected void prngCheck
    (byte[] previousBlock, int previousOffset,
     byte[] currentBlock, int currentOffset, int count)
  {
    if (CryptoJ.getMode() == CryptoJ.NON_FIPS_MODE)
        return;
        
    for (int j = 0; j < count; j++)
      if (previousBlock[previousOffset + j] != currentBlock[currentOffset + j])
        return;

    throw new SecurityException
        ("Continuous Random Number Generation Check failed");
  }

  /**
   * Overrides the default clone to produce a deep clone.
   * Overrides clone in class <code>JSAFE_Object</code>
   * <p>
   * @return A copy of this object.
   * @exception CloneNotSupportedException If the object cannot be cloned.
   */
  public Object clone ()
    throws CloneNotSupportedException
  {
    return (super.clone());
  }

  /**
   * Convert this object into a byte array for serialization.
   * <p>
   * @return This object represented as a byte array.
   */
  byte[] serializeRandom ()
  {
    // For now, just generate random bytes to be used as a seed later.
    return (generateRandomBytes (256));
  }

  /**
   * Convert the input byte array into a <code>JSAFE_SecureRandom</code> object.
   * <p>
   * @param randomAlgorithm The name of the random algorithm to use
   * (e.g., "MD5Random").
   * @param oldRandom A byte array that is the representation of the
   * old random object.
   * @return A new <code>JSAFE_SecureRandom</code> object.
   */
  static JSAFE_SecureRandom deserializeRandom
    (String randomAlgorithm, byte[] oldRandom)
  {
    // For now, just use the byte array as a seed
    JSAFE_SecureRandom returnValue = null;
    try {
      returnValue = (JSAFE_SecureRandom)JSAFE_SecureRandom.getInstance
        (randomAlgorithm, "Java");
      returnValue.seed (oldRandom);
    } catch (NoSuchAlgorithmException algException) {
      // Make sure the returnValue is null.
      returnValue = null;
    }
    return (returnValue);
  }

  /*
   * Copy the values set in this base class to aid in cloning.
   * <p>
   * @param original The object from which to copy values.
   */
  protected void setJSAFELevelValues
    (JSAFE_SecureRandom original)
    throws CloneNotSupportedException
  {
    this.theDevice = original.theDevice;
    this.theDeviceList = original.theDeviceList;
  }

  /**
   * This method clears sensitive data from an object.
   * Although the finalizer clears the data, there is no
   * guarantee the garbage collector will quickly call the
   * finalizer. Allows a user to clear data as soon as
   * possible. After calling <code>clearSensitiveData()</code>,
   * an <code>Init</code> (not a <code>ReInit</code>) method is
   * called to perform other operations with the object.
   */
  public void clearSensitiveData () { }

}
